;; ====================================================================
;; CSWAP.LSP (12-15-97)
;; Color swapping utility dedicated to Fran B. MacMillan,70214,1004
;; For AutoCAD R12, R13, and R14
;; CADvantage Custom Utilities
;; 2 Village Road, Sea Girt, NJ  08750
;; email: CADvantage@compuserve.com
;; FREEWARE
;;=====================================================================
(defun c:CSWAP ( / file fp readdata data mod rewind tbl layer old new)
   (prompt "\nCSWAP.LSP v1.0 (c)1997, John F. Uhden, CADvantage")
   ;| This routine will change the color of all entities in the current
      drawing and its internal block definitions, as well as the color
      of all layers from one color ("old") to another ("new").
      
      Color data must be read from a space-delimited or comma-delimited
      file with the extension ".CSF" (Color Swap File) in the format
      "old,new."  For example:
      
      1,11
      2,12
      255,254
      
      Lines begining with a semicolon ";" are treated as comment lines
      and will be ignored.  Improper data will be ignored and reported
      to the user.  Valid colors are are in the range from 0 to 255.
      Duplicate "old" values will be discarded.  Duplicate "new" values
      are permitted.  Any color that is is not in the data file as an
      "old" color will be ignored, or will be left "as is."
   |;
   ;;-------------------------------------------------------------------
   ;; Function to read the .CSF data file and create a data table (list)
   ;; 
   (defun readdata ( / line i c1 c2)
      (while (setq line (read-line fp))
         (if (/= (substr line 1 1) ";")
            (progn
               (setq i 1 c1 nil c2 nil)
               (while (and (<= i (strlen line))(not c1))
                  (if (member (substr line i 1) '("," " "))
                     (setq c1 (atoi (substr line 1 (1- i)))
                           c2 (atoi (substr line (1+ i)))
                     )
                  )
                  (setq i (1+ i))
               )
               (if (and c1 c2
                        (> c1 -1)(< c1 257)
                        (> c2 -1)(< c2 257)
                        (/= c1 c2)
                        (null (assoc c1 data)))
                  (setq data (cons (cons c1 c2) data))
                  (prompt (strcat "\nImproper data: \"" line "\""))
               )
            )
         )
      )
   )
   ;;---------------------------------------------------------
   ;; Function to modify the color of each and every entity or
   ;; sub-entity only if its color is other than 256 (BYLAYER)
   ;;
   (defun mod (e tell / ent old new)
      (while e
         (if tell (progn (princ "\rProcessing ")(prin1 e)))
         (setq ent (entget e))
         (if (and (setq old (cdr (assoc 62 ent)))
                  (setq new (cdr (assoc old data))))
            (entmod (subst (cons 62 new)(assoc 62 ent) ent))
         )
         (setq e (entnext e))
      )
   )
   ;;-----------------------------------------------------
   ;; Begin by having the user select a Color Swap File...
   ;;
   (if (setq file (getfiled "Select Color Swap File" "" "CSF" 0))
      (if (setq fp (open file "r"))
         (progn
            (readdata)
            (setq fp (close fp))
         )
         (prompt (strcat "\nUnable to open " file " for reading."))
      )
   )
   ;;----------------------------------------------------
   ;; If there is valid data, then process the drawing...
   ;;
   (if data
      (progn
         (setvar "cmdecho" 0)
         (setq rewind 1)
         ;; Modify graphical entities...
         (princ "\n")
         (mod (entnext) 1)
         ;; Modify block definitions...
         (while (setq tbl (tblnext "block" rewind))
            (princ (strcat "\nProcessing block " (cdr (assoc 2 tbl))))
            (if (zerop (logand (cdr (assoc 70 tbl)) 20))
               (mod (cdr (assoc -2 tbl)) nil)
            )
            (setq rewind nil)
         )
         ;; Modify the layers...
         (setq rewind 1)
         (command "_.layer" "_U" "*")
         (while (setq tbl (tblnext "layer" rewind))
            (setq layer (cdr (assoc 2 tbl))
                  old (cdr (assoc 62 tbl))
                  new (cdr (assoc old data))
                  rewind nil
            )
            (princ (strcat "\nProcessing layer " layer))
            (if (and new (< 1 new 256)) (command "_COLOR" (itoa new) layer))
	      )
         (command "")
         (setvar "cmdecho" 1)
      )
      (prompt "\nNo swap data with which to operate.")
   )
   (princ)
)
